import React, {
  PropTypes,
  ViewPropTypes,
  Component,
} from 'react';

import {
  View,
  TextInput,
  Platform,
  StyleSheet,
} from 'react-native';

class AutoTextInput extends Component {
  static propTypes = {
    initHeight: PropTypes.number,
    maxHeight: PropTypes.number,
    inputRef: PropTypes.string,
  }
  static defaultProps = {
    initHeight: 30,
  }

  constructor(props) {
    super(props);
    console.log(this.props.style.height);
    this.state = {
      height: this.props.style.height || this.props.initHeight,
      maxHeight: this.props.maxHeight || this.props.style.height * 3,
    };
  }

  handleHeight = (event) => {
    const curHeight = event.nativeEvent.contentSize.height;
    if (curHeight < this.props.style.height || curHeight > this.state.maxHeight) return;
    this.setState({
      height: curHeight,
    });
    if (this.props.onChange) {
      const callback = this.props.onChange;
      callback(event);
    }
  }

  render() {
    const height = Math.min(this.state.maxHeight, this.state.height);
    return (
      <View>
        <TextInput
          {...this.props}
          multiline
          onChange={this.handleHeight}
          style={[styles.default, this.props.style, { height }]}
          underlineColorAndroid="transparent"
        />
      </View>
    );
  }

}

const padding = Platform.OS === 'ios' ? -5 : 0;
const styles = StyleSheet.create({
  default: {
    textAlignVertical: 'top',
    textAlign: 'left',
    paddingTop: padding,
    paddingLeft: 0,
  },
});

export default AutoTextInput;
